_ = require "lodash"

# Convert Dynamo AttributeValue map object(s) to plain javascript object(s)
#
# @param {Object|Array.<Object>} obj Dynamo AttributeValue map object(s)
# @return {Object|Array.<Object>} plain javascript object(s)
unpackObjectOrArray = (object) ->
  if _.isNull object then return
  if object.Item then object = object.Item
  if object.Items then object = object.Items
  if _.isArray object then return object.map unpackObjectOrArray

  result = {}
  for key, value of object
    if value.S then result[key] = value.S
    if value.N then result[key] = value.N
    if value.SS then result[key] = value.SS
    if value.NS then result[key] = value.NS

  result

# Convert plain javascript object(s) to Dynamo AttributeValue map object(s)
#
# @param {Object|Array.<Object>} plain javascript object(s)
# @return {Object|Array.<Object>} obj Dynamo AttributeValue map object(s)
packObjectOrArray = (object) ->
  if _.isNull object then return
  if _.isArray object then return object.map packObjectOrArray

  result = {}
  for key, value of object
    if _.isString value then result[key] = {S: do value.toString}
    if _.isNumber value then result[key] = {N: do value.toString}
    if _.isArray value
      isNumberArray = _.every value, (n) ->
        _.isNumber n

      value = _.map value, (i) -> do i.toString
      if isNumberArray then result[key] = {NS: value}
      else result[key] = {SS: value}

  result

# Format an plain object to DynamoDB update request format
#
# @param {Object} object - plain javascript object
# @param {String=PUT} action - update action ("PUT","ADD","DELETE")
# @return {Object} Dynamo AttributeValue map object(s)
formatForUpdate = (object, action="PUT") ->
  object = packObjectOrArray object

  result = {}
  for key, value of object
    result[key] =
      Value: value
      Action: action

  result

# Format an plain object to DynamoDB query request format
#
# @param {Object} object - plain javascript object
# @return {Object} Dynamo AttributeValue map object(s)
objectToQuery = (object) ->
  object = packObjectOrArray object
  result = {}

  for key, value of object
    result[key] =
      ComparisonOperator: "EQ"
      AttributeValueList: [value]

  result

module.exports =
  formatForUpdate: formatForUpdate
  objectToQuery: objectToQuery
  packObjectOrArray: packObjectOrArray
  unpackObjectOrArray: unpackObjectOrArray
